; SEGMENT	dat ; MAUSMENU.asm (Usage: MAUSMENU [-]FILENAME)

; Mausmenu simple menu system for DOS - public domain by Eric Auer 1994-2005
; To compile: nasm -o mausmenu.com mausmenu.asm (get nasm at nasm.sf.net)

	org  100h

; Menu file format: 12 lines, each
; text,2*19chars program [* arguments|?string for prompting for args]CR[LF]
; The optional - blocks the ESC/MiddleMouseButton ways of leaving the menu
; The program name # is magic: Instead of a program called #, the shell
; defined by %COMSPEC% is loaded... (comspec feature is new - 2005 by EA)

; Early version 1.2.1994, added "return to text mode" 19.4.1995
; Public domain version and some updates 12/2005 by Eric Auer
; New features 2005: fancy date, directory save/restore, hotkeys, comspec

;***************************************************************************

code:			; the main() of MAUSMENU
	; NOTE: the functions called by main() do not preserve register contents
	call load		; (re)load menu file
	call inst		; reduce memory footprint and relocate stack
	mov ax,3		; 80x25 color text mode...
	int 10h			; init screen
repeat:
	call cls		; clear screen
	call mausan		; enable mouse (if present)
	call titel		; show status bar
	call makbild		; draw all frames
	call showmenu		; draw all button labels
	call maus		; main polling loop
	call mausaus		; disable mouse (if present)
	call cls		; clear screen
	test word [wahl],0fff0h
	jnz ente		; leaving the menu
	jmp exec		; exec does JMP code when it is done
ente:   mov ax,4ceah		; errorlevel ea: "this was mausmenu"
	int 21h			; (used for nested menu instances, mausmenu does not insert a delay on returning to the menu if it finds errorlevel ea...)

;***************************************************************************

load:				; load a menu definition file to our buffer
	mov bl,byte [80h]	; length of command line args
	xor bh,bh
	cmp bl,2
	jb l_l			; too short to make sense
	mov byte [81h+bx],0	; properly terminate string
	mov si,81h
	cld
load2:	lodsb
	cmp al,' '		; leading space
	jz load2
	cmp al,9		; leading tab
	jz load2
	or al,al
	jz l_l2			; no argument
	cmp al,'-'
	jnz load3
	mov byte [locked],1	; set "no exit" flag
	jmp short load2
load3:	mov dx,si
	dec dx			; start of the filename
	mov ax,3d00h		; open for read
	int 21h
	jc l_l2			; failed...
	mov bx,ax		; get the file handle
	mov dx,test1		; our buffer offset
	mov cx,3000		; enough to get 12*((2*19)+filename+args)...
	mov ah,3fh		; Read up to CX bytes from the file...
	int 21h			; (returns number of read bytes, ignored...)
	jc l_l2			; read error...
	push ax
	mov ah,3eh		; close the file again
	int 21h
	pop ax
	cmp ax,100		; oddly short file?
	jb l_l2			; disable lock - maybe a zero menu item file
l_l:    retn			; (could add checks if items are valid :-P)

l_l2:	mov byte [locked],0	; disable "no exit" mode if no menu loaded
	retn


;***************************************************************************

drv:			;  save / restore current drive letter in / from curdrv variable
savedrv:
	mov ah,19h
	int 21h
	mov [curdrv],al
	retn

restdrv:
	mov ah,0eh
	mov dl,[curdrv]
	cmp dl,255
	jz norest
	int 21h
norest: retn

savedir:		; save current directory on current drive
	push ax
	push dx
	push si
	push ds
	mov ax,cs
	mov ds,ax
	mov si,curdir	; location of 64 byte buffer
	mov byte [si],0
	mov dl,0	; use current drive
	mov ah,47h
	int 21h         ; Get Dir (will not include drive letter or initial backslash)
	pop ds
	pop si
	pop dx
	pop ax
	ret

restdir:		; return to previously saved directory
	push ax
	push dx
	push ds
	mov ax,cs
	mov ds,ax
	mov dx,curdir-1	; now with backslash!
	mov ah,3bh
	int 21h         ; Change Dir
	pop ds
	pop dx
	pop ax
	ret


;***************************************************************************

titel:			; initialize the status line
	mov dx,256*22+2		; 23th line, left
	mov si,tit
	mov ch,1ah		; green on blue
	call strtit
	mov dx,256*22+27	; 23th line, middle
	mov si,tit2
	cmp byte [cs:locked],1
	jz etit			; block exit if command line like "MAUSMENU/foo"
	mov ch,1fh		; white on blue
	call strtit
etit:   call uhr		; show the date and time
	retn

strtit: lodsb
	or al,al
	jz estr
	mov ah,ch
	call put
	inc dl
	jmp short strtit
estr:   retn

tit     db 'AuerSoft MAUSMENU 2005',0
tit2    db 'Back: ESC or ctrl-click',0

;***************************************************************************

uhr:	; show date / time in  "Sun Dec 25 2005   22:19:29" style
	mov dx,22*256+53	;  22th line, right
	; Int 21.2a / 1a.4 return date dl.dh.cx, day al
	; Int 21.2c / 1a.2 return time ch:cl:dh
	push dx
	mov ah,2ah
	int 21h			; get date
	mov cx,dx		; high: month, low: day
	pop dx			; now AL is the sunday based day of week
	mov bx,tage		; use this "word list" of day names
	call putname		; ALth 4char element of word list...
	mov al,ch		; month...
	dec al			; make value 0 based!
	mov bx,monat		; use this "word list" of month names
	call putname
	mov al,cl		; day
	call bcd
	mov al,' '
	call putinc
	push dx
	mov ah,2ah
	int 21h			; get date: CX is year
	mov ax,cx
	mov cl,100
	div cl			; now AH is the year and AL is the century
	mov cl,ah
	pop dx
	call bcd		; show century
	mov al,cl
	call bcd		; show year
	mov al,' '
	call putinc
	call putinc
	push dx
	mov ah,2ch
	int 21h			; get time
	mov bl,dh		; second
	pop dx
	mov al,ch		; hour
	call bcd
	mov al,':'
	call putinc
	mov al,cl		; minute
	call bcd
	mov al,':'
	call putinc
	mov al,bl		; second
	call bcd
	retn

putname:			; show 4 chars taken from bx+(4*al)
	push ax
	push bx
	mov ah,0
	add ax,ax
	add ax,ax
	add bx,ax
	mov ax,[bx]
	call putinc
	mov al,ah
	call putinc
	mov ax,[bx+2]
	call putinc
	mov al,ah
	call putinc
	pop bx
	pop ax
	ret

putinc:				; show a char and increment the cursor position
	push ax			; char is in AL, cursor position is in DX
	mov ah,1ah      	; green on blue
	call put
	inc dl
	pop ax
	retn

bcd:   				; convert AL to bcd and display it
	xor ah,ah
	push cx
	mov cl,10		; use 16 if input already IS in bcd...
	div cl
	pop cx
	add ax,'00'
	call putinc
	mov al,ah
	call putinc
	retn

tage    db "Sun Mon Tue Wed Thu Fri Sat "
monat	db "Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec "

;***************************************************************************

makbild:			; draw frames for all 12 buttons (0 to 11)
	mov ax,0
m_k:    push ax
	call calcpos		; location of button on screen: DX
	push dx
	mov bl,[breit]		; button width
	mov bh,[hoch]		; button height
	sub bx,0102h
	mov ah,1ch		; color: green on blue
	clc     ; relative size in BX
	call rahm		; draw frame
	pop dx
	inc dl
	mov al,'F'		; show "F11" style label
	call putinc
	pop ax
	inc ax
	push ax
	call bcd		; show "11" of label...
	pop ax
	cmp ax,12
	jb m_k
	xor dx,dx		; upper left edge
	mov bx,256*25 + 80	; lower right edge ...
	mov ah,12h		; dark green on blue
	clc			; BX is relative to DX
	call rahm		; draw frame around whole screen
	retn

;***************************************************************************

maus:				; main polling loop!
m_an:   call uhr		; show the time and date
	;
	mov ah,84h		; tell that we are idle (kind of - we are in a polling loop)
	int 2ah			; ... or use int 2f.1680, multitasker style ...
	;
	mov ah,1
	int 16h			; keypress waiting?
	jz noesc		; nope, check mouse next
	cmp al,27		; escape pressed?
	jnz noesc2		; nope, but other key...
	cmp byte [cs:locked],1	; esc disabled?
	jz noesc2
okesc:  mov ah,0		;  leaving the menu...
	int 16h			;  take ESC key from buffer
okesc2: mov word [wahl],0ffffh	; store "exit" selection
	retn
	;
totrigger:
	mov ah,0		; consume the key...
	int 16h			; (remove it from the buffer)
	mov ax,[wahl]
	jmp buttontrigger
	;
noesc2:				; not ESC but maybe another hotkey, AL ASCII, AH scancode
	mov word [wahl],11-1	; F1..F12 hotkeys added 12/2005
	cmp ah,57h		; F11?
	jz totrigger		; F11 triggered!
	mov word [wahl],12-1
	cmp ah,58h		; F12?
	jz totrigger		; F12 triggered!
	cmp ah,3bh		; F1?
	jb noesc3
	cmp ah,44h		; F10?
	ja noesc3
	sub ah,3bh
	mov al,ah
	mov ah,0
	mov [wahl],ax
	jmp short totrigger	; hotkey triggered!
noesc3: mov ah,0
	int 16h			; take keyboard key from buffer
noesc:	call ismaus
	jc m_an			; keyboard-only operation...
	;
	mov ax,3		; get mouse position and mouse-buttons
	int 33h
	and bx,7
	jz m_an			; none pressed? then LOOP around, keep polling...
	cmp bl,4        ; added 4/1995: middle mouse button leaves, too...
	jz m2aus
	mov ah,2
	int 16h			; get shift status
	test al,4		; CTRL pressed?
	jz m_aus		; else: normal menu selection click
m2aus:  mov ax,3		; wait until all buttons are released again
	int 33h
	test bx,7
	jnz m2aus
	cmp byte [cs:locked],1
	jnz okesc2	; mouse exit, if allowed...
m2an:   jmp m_an
	;
m_aus:  mov ax,3        ; Get mouse position (CX,DX) and mouse-buttons (BX)
	int 33h
	test bx,7
	jnz m_aus       ; poll until all mouse-buttons released
	 mov ax,cx
	 mov cl,3       ; convert from "virtual pixel" to "char" coordinates
	 shr ax,cl
	 shr dx,cl
	 mov cx,ax
	sub cl,[lioben]		;  left edge of left button column...
	jc m_an
	sub dl,[obenli]		; upper edge of upper button row...
	jc m_an
	mov ax,cx
	div byte [breit]	; calculate button column number...
	add ah,2
	cmp ah,[breit]		; button width
	jnb m_an
	mov cl,al
	mov ax,dx
	div byte [hoch]		; calculate button row number...
	inc ah
	cmp ah,[hoch]		; button height
	jnb m2an
	mov dl,3		; three buttons per row...
	mul dl
	add al,cl		; now AX is the number of the selected button
	cmp al,12
	jnb m2an		; out of range click - ignore
buttontrigger:
	mov word [wahl],ax
	 push ax
	 call calcpos		; calculate button position
	 mov bl,[breit]		; frame width
	 mov bh,[hoch]		; frame height
	 sub bx,0102h		; ...
	 mov ah,1fh		; white on blue
	 clc			; BX is relative size
	 call rahm		; draw highlighted frame
	 pop ax
	retn

wahl    dw 0    ; number of the selected button

;***************************************************************************

calcpos:			; calculate position DL DH for button number AX
	push ax
	and ax,15
	mov dl,3		; 3 button columns
	div dl
	mov dl,ah
	push ax
	mov al,[breit]		; button width
	mul dl
	mov dl,al
	pop ax
	mul byte [hoch]		; button height
	add al,[obenli]		; vertical offset
	mov dh,al
	add dl,[lioben]		; horizontal offset
	pop ax
	retn

lioben  db 5			; left edge of left buttons
obenli  db 2			; upper edge of upper buttons
breit   db 23			; button width, including frame
hoch    db 5			; button height, including frame
brtext  equ 19			; button label width per line

;***************************************************************************

mausan:			; initialize mouse and show mouse pointer
	call ismaus		; mouse present??
	jc nomaus
	mov ax,0	        ; initialize
	int 33h
	or ax,ax		; driver will return AX = -1
	jz nomaus		; else:	 no mouse found
	mov ax,1		; activate mouse cursor
	int 33h
	mov ax,0ah		; define mouse cursor shape as char...
	mov bx,0		; ...not as block...
	mov dx,2400h		; XOR mask for color,char
				; we use: change color, keep char
	mov cx,00ffh		; AND mask for color,char
				; we use: zap color, keep char
	int 33h
	mov ax,4	        ; set cursor position
	xor cx,cx
	xor dx,dx
	int 33h
	mov ax,1dh		; set cursor page
	xor bx,bx
	int 33h
m_a:    mov ax,3		; get mouse position cx dx, button bx
	int 33h
	test bx,7
	jnz m_a			; wait until all mouse buttons released
nomaus: retn

;***************************************************************************

ismaus:			; return CY if no mouse is present. modifies AX
	push es
	push bx
	mov ax,3533h		; check mouse interrupt vector
	int 21h
	mov ax,es
	or bx,ax
	mov al,byte [es:bx]	; check handler
	or bx,bx		; check pointer
	pop bx
	pop es
	jz nomo			; oops...
	cmp al,0cfh		; iret? oops...
	jz nomo
	clc
	retn
nomo:	stc
	retn
	

mausaus:
	push ax
	call ismaus
	jc noma
	mov ax,2        ; hide mouse pointer
	int 33h
noma:	pop ax
	retn

;***************************************************************************

rahm:    ; Draw a frame from DL DH to BL BH (if CY) or draw a frame
		; from DL DH in size BL BH (if carry flag off / NC). Color AH.
	push ax
	push cx
	push bx
	jnc no_re
re:     sub bx,dx
no_re:
	xor cx,cx

	mov cl,bl       ; d_x
	dec cx
	dec cx
	mov al,''
	call put
	inc dx  ; dl
	mov al,''
ei:      call put
	 inc dx  ; dl
	 loop ei
	mov al,''
	call put

	inc dh
	mov cl,bh       ; d_y
	dec cx
	dec cx
	mov al,''
zw:      call put
	 inc dh
	 loop zw

	mov al,''
	call put
	dec dx  ; dl
	mov cl,bl       ; d_x
	dec cx
	dec cx
	mov al,''
dr:      call put
	 dec dx ; dl
	 loop dr
	mov al,''
	call put
	dec dh

	mov cl,bh       ; d_y
	dec cx
	dec cx
	mov al,''
vi:      call put
	 dec dh
	 loop vi

	pop bx
	pop cx
	pop ax
	retn

;***************************************************************************

put:    ; put char AL at position DL DH in color AH (write to screen buffer)
	push bx
	push ds
	push ax
	mov al,2*80		; assume fixed screen width
	mul dh
	mov bx,dx
	shl bx,1
	xor bh,bh
	add bx,ax
	 mov ax,0b800h		; assume color text mode
	 mov ds,ax
	pop ax
	mov [bx],ax
	pop ds
	pop bx
	retn

;***************************************************************************

cls:		; guess what - clear the screen...
	push es
	push ax
	push di
	push cx

	mov ax,0b800h	; assume color text mode
	mov es,ax       ; target pointer uses ES
	mov ax,1e20h    ; space, (yellow on) blue
	mov cx,80*25    ; assume fixed screen size
	xor di,di
	cld
	REP STOSW

	mov ah,2        ; set cursor position to upper left
	xor dx,dx
	mov bh,0
	int 10h

	pop cx
	pop di
	pop ax
	pop es
	retn

;***************************************************************************

show:
showmenu:			; draw the whole menu of 3x4 buttons
	xor ax,ax
sw:     mov si,test1	; menu buffer location...
	push ax
	call calcpos		; get the screen location of a menu item
	add dx,0101h		; make coordinates 1 based
	 push dx
	 push ax
	call find		; get the buffer offset of that item
	 pop ax
	 pop dx
	jc sn
	call disp2		; show the item on screen
	pop ax
	jnc oksn
	push ax			; if item has no label, remove it
	call calcpos
	mov bl,[breit]		; frame width
	mov bh,[hoch]		; frame height
	sub bx,0102h		; screen position of the frame
	mov ah,11h		; invisible frame (remove frame)
	clc			; select "delta mode"
	call rahm		; draw frame
	pop ax
oksn:   inc ax
	cmp ax,12		; all items done?
	jb sw			; else iterate
	retn
sn:     pop ax
	retn
nixi:
	stc
	retn

disp2:				; show a menu item label text
	push si			; see also the disp function
	mov cx,2*brtext		; fixed width. first pass: check string
ci:     lodsb
	call eof
	loopnz ci
	or cx,cx
	pop si
	jnz nixi		; NZ means "text ended to early" here
	mov cl,2*brtext		; start second pass
dxi:    lodsb
	mov ah,1eh      ; yellow on blue
	call put
	inc dl
	cmp cl,brtext+1		; label is wrapped into 2 lines...
	jnz d_i
	sub dl,brtext		; "CR" inside label box area
	inc dh			; "LF" inside label box area
d_i:    loop dxi
	clc
	retn


;***************************************************************************

exec:			; run the selected program
	mov ax,cs
	mov es,ax
	mov ds,ax
	call savedrv		; save drive letter
	mov word [inf+4],cs	; generate far pointer
	; call inst	; reduce memory footprint and relocate stack
	mov ax,[wahl]   ; fetch the menu item selection
	and ax,0fh
	mov si,test1	; location of our menu buffer
	call find		; seek to the Nth line in the buffer
	jc xxx			; line not found...
	call disp		; seek over the label of the menu item line
	jc bye2
	call gfil		; fetch the command file name
	jc bye2
	call savedir		; save directory
	call chdir		; jump to directory / drive of the command
	jc xxx
	call parm		; process command arguments
	; sets CY if no argument given...
	call m_sage	; added 4/95: the "click to return newsflash"
	call execut		; actually run the command
	cld
	jnc bye
xxx:    mov al,7		; beep
	call tty
	jmp short bye2
bye:    cmp al,0eah		; skip "wait for mouse" if errorlevel ea...
	jz bye2
	mov si,retmsg
rettype:
	lodsb
	or al,al
	jz rettyped
	call tty
	jmp short rettype
	;
rettyped:
	call mausan		; enable mouse, if found
byye:   call ismaus
	jc bye2a		; none - only wait for keyboard
	mov ah,1
	int 16h			; key waiting?
	jnz bye2a
	mov ax,3
	int 33h			; wait for mouse button press
	test bx,7
	jz byye
	call mausaus		; disable mouse again
	jmp short bye2
bye2a:				; consume key... or wait for key!
	mov ah,0
	int 16h			; fetch keypress
bye2:   mov ax,3
	int 10h			; reset screen to text mode
	call restdrv		; restore drive letter
	call restdir		; restore directory
	jmp repeat		; back to main()

m_sage:         ; added 1995:	 flash a "click to return" message on the screen
%if 0
	; pusha			;  requires 186+ CPU, and actually we may destroy regs anyway...
	mov cx,1
	call seconds
	call cls
	mov si,bbye ; pointer to the message
	mov dx,0c0dh    ; position (dl=x dh=y)
	mov ch,1ch      ; color: green on blue
	call strtit
	mov cx,2
	call seconds
	call cls
	; popa			; requires 186+ CPU
%endif
	retn

%if 0
seconds:        ; wait until the second changes. do it (CX and 31)+1 times...
	and cx,1fh		; just if somebody sent us a foolishly big CX
	inc cx
	push dx
eizwse: push cx
	mov ah,2ch
	int 21h			; get system time, seconds in DH
	pop cx			; int 21.2c changes CX and DX, so protect CX
	mov ch,dh
ezs:    push cx
	mov ah,2ch
	int 21h
	pop cx
	cmp ch,dh		; second changed yet?
	jnz ezs			; else keep waiting
	mov ch,0
	loop eizwse		; next iteration
	pop dx
	retn			; AX, CX, DX destroyed
%endif

inst:				; adjust stack and memory allocation
	 pop bp			; figure out who called us
	mov bx,[test1+3000]       ; put stack at end of menu buffer
	and bx,0fffeh
	add bx,128
	mov sp,bx
	mov cl,4
	shr bx,cl
	inc bx
	mov ah,4ah      ; resize our own memory block, give other programs more RAM
	int 21h
	 push bp		; prepare to return to the caller
	retn

find:				; go to the AXth line in a buffer
	inc ax
	cld
	dec ax
	jz ok
	mov cx,ax
b:      lodsb
	call eof		; seek to the end of a line
	jnz b
	call eef		; also end of the buffer? then we did not find a string...
	jz nix
	 cmp byte [si],10	; skip over LF (CR LF only counts as 1 linebreak)
	 jz b
	loop b
ok:     clc
	retn
nix:    stc
	retn

disp:				; skip over menu item label text
	push si			; (alternative:	could highlight text here)
	mov cx,2*brtext		; label has fixed size
c:      lodsb
	call eof
	loopnz c		; done after N columns or at end of line
	or cx,cx
	pop si
	jnz nix
	add si,2*brtext
	clc
	retn

gfil:				; fill filename buffer
	mov di,fil
	mov byte [di],0
	mov cx,64
e:      lodsb
	call e_f		; check for end of filename chars
	jz gg
	mov [cs:di],al		; copy char
	inc di
	loop e
gg:     cmp cx,64		; buffer still empty?
	jz nix			; does "stc ret" ("not a menu item")
	mov ah,al		; remember the end-of triggering char
	mov byte [cs:di],0	; terminate buffer
	cmp word [fil],'#'	; only "#" (followed by 0) in buffer?
	jz comsp		; then replace the name by %COMSPEC%...
	retn

comsp:	push ds			; new 12/2005, taken from FreeDOS CALLVER
	push ax
	push si
	mov di,fil		; fill buffer with comspec...
	mov ax,[ds:2ch]		; get environment segment from PSP
	xor si,si
trycomsp:
        lodsw
        and ax,0dfdfh   ; upcase
        cmp ax,"CO"
        jnz skipcomsp
        lodsw
        and ax,0dfdfh   ; upcase
        cmp ax,"MS"
        jnz skipcomsp
        lodsw
        and ax,0dfdfh   ; upcase
        cmp ax,"PE"
        jnz skipcomsp
        lodsw
        and al,0dfh     ; upcase (only the last "c" of "comspec"...)
        cmp ax,"C="
        jnz skipcomsp
	; HERE we found the comspec value :-) copy (no overflow check!)...
copycomsp:
	lodsb
	mov [cs:di],al
	inc di
	or al,al	; was that the trailing 0?
	jz comspdone
	jmp short copycomsp
	;
skipcomsp:
        lodsb
        or al,al
        jnz skipcomsp   ; skip rest of var until 0 byte
        cmp [ds:si],al  ; next byte is 0, too?
        jz owncomsp     ; bad luck, no COMSPEC found
        jmp trycomsp	; try next env variable
	;
owncomsp:		; use fallback...
	mov ax,cs
	mov ds,ax
	mov si,comspec	; our own default value, that is...
	jmp short copycomsp

comspdone:
	pop si
	pop ax
	pop ds
	ret

chdir:				; go to drive / directory found in filename
	push si
	push ax
	mov ax,word [fil]	; get first 2 chars of filename
	cmp ah,':'		; test if 2nd char is ":"
	jnz nowech		; if no, do not change drive
	mov dl,al		; if yes, 1st char is drive letter
	and dl,1fh
	cmp dl,26
	jnb nowech
	dec dl          ; a: = 0 and so on
	mov ah,0eh
	int 21h         ; Change Drive
nowech: mov di,fil
	xor al,al
	mov cx,64
	repnz scasb	; find end of string
	std		; select backwards search direction
	mov al,'\'
	mov ah,cl
	mov cx,64
	sub cl,ah
	repnz scasb     ; find last backslash
	inc di
	cld		; return to forwards search direction
	cmp byte [di],'\'
	clc
	jnz nchdir
	mov [di],ch  ; 0
	mov dx,fil
	mov ah,3bh
	int 21h         ; Change Dir
	mov byte [di],'\'
nchdir: pop ax
	pop si
	retn

parm:				; process arguments for to be called arguments
	mov di,par
	mov [cs:di],al
	inc di
	xor cl,cl
	cmp ah,'?'		; show a prompt - ask the user for arguments
	jz inp
	cmp ah,'*'
	jnz pp			; no arguments given
pme:    lodsb			; get arguments from menu definition
	call eof
	jz pp
	inc cl
	cmp cl,126
	ja pp
	mov [cs:di],al
	inc di
	jmp short pme
	;
pp:     mov byte [cs:par],cl
	mov byte [cs:di],13
	retn
	;
inp:    lodsb			; get args from user
	call eof
	jz ii
	call tty
	jmp short inp
ii:	call crlf
	mov dx,gpar
	mov ah,10		; fetch string from user
	int 21h
	call crlf
	retn

execut:				; run another program
	; pusha			; requires 186+ CPU
	; actually no normal registers have to be preserved by execut anyway :-p
	push ds
	push es
	mov [s_ptr],sp
	mov word [s_ptr+2],ss
	 mov dx,fil
	 mov bx,inf
	 mov ax,4b00h
	 int 21h		; call a program
	cli
	mov sp,[cs:s_ptr]
	mov ss,[cs:s_ptr+2]
	sti
	pop es
	pop ds
	; popa			; requires 186+ CPU
	jnc xx
	retn     ; Error
xx:     mov ah,4dh
	int 21h ; fetch errorlevel from called program
	retn

e_f:				;  test if AL is a char which ends the program name string
	cmp al,'*'
	jz ee
	cmp al,'?'
	jz ee
eof:    cmp al,10
	jz ee
	cmp al,13
	jz ee
eef:    cmp al,26
	jz ee
	or sp,sp		; return NZ
ee:     retn			; return ZF if any of the eof chars

tty:				; output a char via BIOS
	push ax
	push bx
	mov ah,0eh		; teletype output
	mov bx,1eh		; yellow on blue, page 0
	int 10h
	pop bx
	pop ax
	retn			; all registers preserved

crlf:				; wrap to next line
	push ax
	push bx
	mov al,13
	call tty
	mov al,10
	call tty
	pop bx
	pop ax
	retn			; all registers preserved


;***************************************************************************

curdrv  db 255		; current drive, -1 for not set yet
	db "\"		; backslash as prefix for curdir for restdir
curdir  db "c:\dos",0
	times 64 db 0	; current directory
fil     times	128	db 0
gpar    db 127		; header for program arguments
par     db 0		; size of string
	times	127	db 0	; actual string

inf     dw 0	; segment with environment variables
	dw par,42	; the 42 will be replaced by our code segment
	dw 0,0          ; first FCB pointer
	dw 0,0          ; second FCB pointer

s_ptr	dw 0,0		; we will store our SS:SP here during exec
locked	db 0		; 1 if ESC key blocked

comspec	db 'c:\command.com',0	; used if no COMSPEC environment variable found
%if 0
bbye	db 'Press any mouse button after program completion to return to menu',0
%else
retmsg	db '[Press a key or click mouse to return]',0
%endif

	   ;12345678901234567891234567890123456789filename.com...
test1   db 13,10                                 ;v;
test2   db 'This program takes a menu file as arg.',13,10
test3   db 13,10

test4   db 'DOS SHELL:         calls COMMAND.COM  #?Type /c doscommand or just hit enter for a shell...',13,10
test5   db 13,10
test6   db 13,10

test7   db 13,10
test8   db 13,10
test9   db 'Set time:          calls COMMAND.COM  #*/c time',13,10

test10  db 13,10
test11  db '12345678901234567891234567890123456789',13,10
test12  db 13,10
	db 26

	db '(c) AuerSoft '

;***************************************************************************

END
